# Authors: David Whitlock <alovedalongthe@gmail.com>
# A simple text analysis tool
# Copyright (C) 2013-2014 David Whitlock
#
# Drat is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Drat is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Drat.  If not, see <http://www.gnu.org/licenses/gpl.html>.

import argparse
import sys
import requests
import zipfile
import xml.etree.ElementTree as ET
from html.parser import HTMLParser
from . import analysis

usage_info = """The file, or url, you have chosen will be compared with a list of
common English words, and a report of the results will be saved in the current working directory.
You can also provide a list of url links (with each link on a separate line) written in a text file."""

class ArgsHandler(object):
    """Handle the command line arguments."""
    def __init__(self, args):
        self.args = args
        if not args.infile:
            with sys.stdin as f:
                self.args.infile = [arg.strip() for arg in f]

    def args_handler(self):
        for arg in self.args.infile:
            if arg.startswith('http'):
                self.check_url(arg)
            else:
                self.check_file(arg)

    def check_url(self, arg):
        response = requests.get(arg)
        html = response.text
        url_reader = HtmlParser()
        url_reader.feed(html)
        data = ''.join(url_reader.text)
        check = analysis.Checktext(arg, self.args.wlist, self.args.verb, False)
        check.run_check(data.encode('utf-8'))

    def check_file(self, arg):
        exts = ('.docx', '.odt', '.ods', '.odp')
        if arg.endswith(exts):
            doc_reader = DocParser(arg)
            data = doc_reader.get_doctype()
        else:
            with open(arg, 'rb') as f:
                data = f.read()
        check = analysis.Checktext(arg, self.args.wlist, self.args.verb, False)
        check.run_check(data)

class HtmlParser(HTMLParser):
    """Parse urls."""
    def __init__(self):
        HTMLParser.__init__(self)
        self.body = False
        self.get_data = True
        self.text = []
        self.sentences = 0

    def handle_starttag(self, tag, attrs):
        if tag == 'body':
            self.body = True
        if tag == 'p':
            self.get_data = True
        if tag == 'script' or tag == 'a':
            self.get_data = False

    def handle_endtag(self, tag):
        if tag == 'body':
            self.body = False
        if tag == 'p':
            self.get_data = False
        if tag == 'script' or tag == 'a':
            self.get_data = True

    def handle_data(self, data):
        if self.body and self.get_data:
            self.text.append(data)

class DocParser(object):
    """Parse docx and odf files."""
    def __init__(self, infile):
        self.infile = infile

    def get_doctype(self):
        if self.infile.endswith('.docx'):
            docid = 'word/document.xml'
        else:
            docid = 'content.xml'
        zfile = zipfile.ZipFile(self.infile)
        body = ET.fromstring(zfile.read(docid))
        text = '\n'.join([et.text.strip() for et in body.iter() if et.text])
        return text.encode('utf-8')

def main():
    parser = argparse.ArgumentParser(description='Text analysis tool', prog='drat', epilog=usage_info)
    parser.add_argument('infile', type=str, nargs='*', help='Name of file(s), or url(s) you want checked.')
    parser.add_argument('-v', '--verbose', dest='verb', action='store_true', help='Print more detailed information.')
    parser.add_argument('-w', '--wordlist', dest='wlist', type=str,
            nargs='?', help='Name of additional wordlist(s) you want to use.')
    args = parser.parse_args()
    handler = ArgsHandler(args)
    handler.args_handler()



# Authors: David Whitlock <alovedalongthe@gmail.com>
# A simple text analysis tool
# Copyright (C) 2013-2014 David Whitlock
#
# Drat is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Drat is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Drat.  If not, see <http://www.gnu.org/licenses/gpl.html>.

import os
import json
import textwrap
from collections import Counter

class Checktext(object):
    def __init__(self, name, wlist, verb, web):
        self.name = name
        self.verb = verb
        self.web = web
        self.base_dir = os.path.abspath(os.path.dirname(__file__))
        self.load_common(wlist)
        self.load_dale_chall()

    def load_common(self, wlist):
        """Create the dictionary of common words."""
        self.com_dict = os.path.join(self.base_dir, 'dicts', 'EN_vocab.json')
        with open(self.com_dict) as words_file:
            data = json.load(words_file)
        self.common_words = set(data)
        if wlist:
            new_words = self.load_wlists(wlist)
            if new_words:
                new_dict = {word.strip() for word in new_words.splitlines()}
                self.common_words.update(new_dict)

    def load_dale_chall(self):
        """Create the dictionary of words, and grade dictionary, for the Dale-Chall readability test."""
        self.dale_chall_dict = os.path.join(self.base_dir, 'dicts', 'dale_chall.json')
        with open(self.dale_chall_dict) as words_file:
            data = json.load(words_file)
        self.dale_chall_words = set(data)
        self.dale_chall_grade = {4.9: 'Grade 4 and below', 5.9: 'Grades 5-6', 6.9: 'Grades 7-8',
                7.9: 'Grades 9-10', 8.9: 'Grades 11-12', 9.9: 'Grades 13-15'}

    def load_wlists(self, wlist):
        new_words = ''
        try:
            with open(wlist) as f:
                new_words = f.read()
        except:
            for wl in wlist.split():
                with open(wl) as f:
                    new_words += f.read()
        return new_words

    def run_check(self, data):
        """Check for uncommon words and difficult words in file."""
        sentences = data.count(b'.') + data.count(b'!') + data.count(b'?') or 1
        punc = b'!"#$%&\'()*+,-./:;<=>?@[\\]^_`{|}~0123456789'
        data = data.translate(bytes.maketrans(punc, b' ' * len(punc)))
        words = Counter(data.decode('utf-8').lower().split())
        uniq_len = len(words)
        self.total = sum(words.values())
        uncommon = Counter({word: count for word, count in words.items() if word not in self.common_words})
        dchall_set = Counter({word: count for word, count in words.items() if word not in self.dale_chall_words})
        diff_count = sum(dchall_set.values())
        dale_chall_score = round(self.dale_chall(diff_count, sentences), 1)
        self.fmt_output(uniq_len, uncommon, dale_chall_score)

    def dale_chall(self, diff_count, sentences):
        """Calculate Dale-Chall readability score."""
        pdw = diff_count / self.total * 100
        asl = self.total / sentences
        raw = 0.1579 * (pdw) + 0.0496 * asl
        if pdw > 5:
            return raw + 3.6365
        return raw

    def fmt_output(self, uniq_len, uncommon, dale_chall_score):
        uncom_len = len(uncommon)
        for key in self.dale_chall_grade:
            if dale_chall_score < key:
                self.read_grade = self.dale_chall_grade[key]
                break
        else:
            self.read_grade = 'Grade 16 and above'
        self.message = 'Report for {}.\n'.format(self.name)
        self.message += 'There are {:d} uncommon words in this text.\n'.format(uncom_len)
        self.message += 'This is out of a total of {:d} unique words.\n'.format(uniq_len)
        self.message += 'The Dale-Chall readability score is {:.1f} ({}).\n'.format(dale_chall_score, self.read_grade)
        if self.verb:
            self.message += 'The following {:d} words are not in the list of common words:\n'.format(uncom_len)
            for item in uncommon.most_common():
                self.message += '{}: {}  '.format(item[0], item[1])
        if not self.web:
            for line in self.message.splitlines():
                print(textwrap.fill(line, width=120))



# Authors: David Whitlock <alovedalongthe@gmail.com>
# A simple text analysis tool
# Copyright (C) 2013-2014 David Whitlock
#
# Drat is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Drat is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Drat.  If not, see <http://www.gnu.org/licenses/gpl.html>.

from setuptools import setup

with open('README.rst') as f:
    long_description = f.read()

setup(
    name='drat',
    version='0.1.8',
    author='David Whitlock',
    author_email='alovedalongthe@gmail.com',
    url='https://github.com/riverrun/drat',
    description='A reading text analysis tool',
    long_description=long_description,
    license='GPLv3',
    packages=['drat'],
    include_package_data=True,
    zip_safe=False,
    platforms='any',
    install_requires=['requests'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Topic :: Education',
        'Topic :: Office/Business',
    ],
    entry_points={
        'console_scripts': [
            'drat = drat.application:main',
            ]
        },
)
